#!/tvbin/tivosh 

# dailymail_jazz.tcl
# email TiVo status and info
#
# Original Baseline Version 0.3 (29/11/03) by Stuart Anderton (sanderton at tivocommunity.com)
# Some procs borrowed from TiVoWeb as you can't call them by reference
# Just jazzed up a little by Chris Stennett (tivo-onion at tivocommunity.com)
#
# Change Log:
# -----------
#	01 Oct 05 ---- v1.0-----------------------
#	- First Formal Release
#	02 May 06 ---- v1.01-----------------------
#	- fixes included that have been explained in the TiVo Community Forums
#       - up to this date.
#
#
# 04 Sep 2011 v1.02 (released 14 Aug 2012) [spitfires] Complete rewrite and restructure of program code
#			- Fixed cron decode (only worked with very simple schedule entries)
#			- Use new tracker (3.20)
# 		- Fixed checking of SMART status
# 		- Fixed programme space usage in ToDo display
# 		- Fixed bug where it would display "it is planned to be recorded at a later time" instead of "it is already recorded and available" if there was e.g. 2 recordings upcoming, and it had already been recorded (it should check the Now Showing list *before* the ToDo list)
# 		- Removed replicated & redundant code (Average runtime reduced from 10 minutes to 3 minutes)
#     - New config params:
#         (2.1.6) Tivowebplus version - tivowebplusversion
#         (3.8.1) Missing Cron List - cronlistmissing
#         (2.2.6.4) Plain text username & password - username_ascii and (2.2.6.5) password_ascii
#     - Note for AltEPGUK users: certain functions require TMSIDs, please ensure you have selected this option in your AltEPG account
#
# 17 Aug 2012 v1.02A [spitfires] Incorporate [millsb] changes for AltEPG e-mail service
# 19 Dec 2013 v1.03 Better integration with AltEPG email service
#                           Fix bug in Tracker detection
#
#



# #######################################################################################################
# VERSION INFORMATION SET HERE:
set software_version "1.03"
set software_release_date "19 December 2013"


# error stack global variable for error handling
global errorInfo



# #######################################################################################################
# PROCEDURES

# debug procedure
proc dmdebug {level text {withtime true} {withnewline true}} {
	if {![info exists ::debuglvl]} { set ::debuglvl 1 }
	if {$::debuglvl >= $level} { 
		if {$withtime != "false"} {
			puts -nonewline "[clock format [clock seconds] -format {%H:%M:%S} -gmt true]-" 
		}
		if {$withnewline == "true"} {
			puts $text
		} else {
			puts -nonewline $text
		}
	}
}

# log error
proc dmerrorinfo {level errorInfo} {
	if {$::debuglvl > $level} { 
		puts "----------------------------------------------------------------"
		puts "$errorInfo"
		puts "----------------------------------------------------------------"
	}
}

proc max {a b} { if {$a<$b} {return $b} else {return $a} }

proc strim str {
  #regsub -all {^\{|\}$} $str "" str
  if {([string index $str 0]=="\{") && ([llength $str]==1)} {
    return [string range $str 1 [expr [string length $str]-2]]
  }
  return $str
}

proc defaultval {val1 val2} {
  if { $val2 != "" } {
    return $val2
  } else {
    return $val1
  }
}

proc nl2br str {
	regsub -all "\n" $str {<br />} str
	return $str
}
	
proc nl2crnl str {
	regsub -all "\n" $str {\r\n} str
	return $str
}

proc nl2space str {
	regsub -all "\r?\n" $str { } str
	return $str
}

proc replacechar {str from to} {
	regsub -all "$from" $str "$to" str
	return $str
}

proc spacetounderscore str {
	return [replacechar $str " " _]
}

proc printarray array {
	upvar $array arr
	foreach {key value} [array get arr] {
		 puts "Key: $key Value: $value" 
	}
}
	
proc ftime { secs format } {
	if {$secs == 0} { return "" }
  return [clock format [expr $secs + $::tzoffset] -format $format]
}

proc displaylocaldatetime secs {
	return [ftime $secs $::datetimeformat]
}

proc displaylocaldate secs {
	return [ftime $secs $::dateformat]
}

proc displaylocalday secs {
	return [ftime $secs $::dayformat]
}

proc displaylocalshortdatetime secs {
	return [ftime $secs $::shortdatetimeformat]
}

proc displaylocaltime secs {
	return [ftime $secs $::timeformat]
}

proc recqualitytext {quality {case camel}} {
	array set quals { 0 Basic 40 Medium 75 High 100 Best default Best }	
	switch $case {
		"camel"	{ return $quals($quality) }
		"lower"	{ return [string tolower $quals($quality)] }
		"upper"	{ return [string toupper $quals($quality)] }
	}
}
						
proc imageref {filename} {
	if { $::embeddedimages } {
		## build a list of the images we've used
		if {[lsearch $::imagenamesused $filename] == -1} {
			lappend ::imagenamesused $filename
		}
	}
	if { $::embeddedimages } {
		return "cid:$filename.$::uniquecontentid@$::domain"
	} else {
		return "$::dmailGraphicsURL/$filename"
	}
	

}

# used to display the 'boxes' on the space bar graphic
proc spacegraphic {graphic size colour alt {pos ""}} {
	if {$size > 0} {
		if {$::graphicson} {
			if {$::popuptextrequired} {
				writehtml "<img src=\"[imageref $graphic]\" border=\"0\" width=\"$size\" height=\"12\" vspace=\"0\" hspace=\"0\" title=\"$::drivetext\" alt=\"$alt\" onclick=\"alert('$::drivepoptext')\" />" 1
			} else {
				writehtml "<img src=\"[imageref $graphic]\" border=\"0\" width=\"$size\" height=\"12\" vspace=\"0\" hspace=\"0\" title=\"$::drivetext\" alt=\"$alt\" />" 1
			}
		} else {
			if {$pos == "begin"} {
				writehtml "\[" 1
			} elseif {$pos == "end"} {
				writehtml "\]"
			} else {
				set nongraphiclowsizecount 0
				catch { set nongraphiclowsizecount [ expr round(($size / $::multipletime) / 2) ] }
				if {$size != 0 && $nongraphiclowsizecount == 0} {  set nongraphiclowsizecount 1 }
				if {$nongraphiclowsizecount > 0} {
					set nongraphiccount 1
					writehtml "<span style='color:$colour'>" 1
					while {$nongraphiccount <= $nongraphiclowsizecount} {
						writehtml "&#9632;" 1
						incr nongraphiccount
					}
					writehtml "</span>" 1
				}
			}				
		}
	}
}	
			
			
# ##############################################
# save results as an html file

proc openhtml {} {
	if { $::htmlfilefd == 0 } {
		if { [ catch { set ::htmlfilefd [open $::htmlfilename "w"] } ] } {
		} else {
			puts $::htmlfilefd ""
			puts $::htmlfilefd "<html>"
			puts $::htmlfilefd "<head>"
			puts $::htmlfilefd "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">"
			puts $::htmlfilefd "<title>TiVo Daily Status E-mail</title>"
			outputStyleSheet $::htmlfilefd
			puts $::htmlfilefd "</head>"
			puts $::htmlfilefd "<body>"
		}
	}
}

proc writehtml {message {nonewline 0}} {
	if { $::htmlfilefd == 0 } {	openhtml }
	if { $::htmlfilefd > 0 } {
		if {$nonewline == 0} {
			puts  $::htmlfilefd $message
		} else {	
			puts -nonewline $::htmlfilefd $message
		}
		flush $::htmlfilefd
	}
}

proc closehtml {} {
	if { $::htmlfilefd > 0 } {
		puts $::htmlfilefd "</body>"
		puts $::htmlfilefd "</html>"
		flush $::htmlfilefd
		close $::htmlfilefd
		set ::htmlfilefd 0
	}
}
		


# #######################################################################################################
# Define stylesheet 

proc outputStyleSheet {chan} {
	set css "<style type='text/css'>"
	if { [ catch { set fd [open $::pathname/dailymail_jazz.css r] } ] } {
			dmdebug 1 "ERROR - CAN'T OPEN CSS FILE FOR READING"			
			dmerrorinfo 0 $::errorInfo
	} else {
		set filecss [read $fd]
		close $fd
		eval append css "\"$filecss\""				;# make sure you have double double quotes to eval the css read in
		unset filecss
	}
	append css  "</style>"
	puts $chan [nl2space $css]
}		;# END proc outputStyleSheet





# #######################################################################################################
# output the specified section

proc outputSection {index} {

	global errorInfo

	set sectionHeading ""
	set sectionWithData ""
	set sectionWithoutData ""
	set sectionMessage ""
	set itemsFound 1
	
	if {$::yankspeak} {
		set ::program_word "rogram"
		set ::expiry_word "xpiry"
		set ::organize_word "rganize"
	} else {
		set ::program_word "rogramme"
		set ::expiry_word "xpiration"
		set ::organize_word "rganise"
	}

	dmdebug 2 "generating output section >>$index<<"
								
	# ###################################################################################################
	# # TODO LIST CODE
	#
	if { $index == $::dmailToDoList } {
	
		dmdebug 2 "generating to-do list"
		
		writehtml "<br><span class=\"SectionHeading\">To Do</span><br>"
		writehtml "<table><tr><td class=\"header\">"
		writehtml "The following p${::program_word}s are scheduled to record in the next $::dmailHoursahead hours: "
		writehtml "</td></tr></table>"
		writehtml "<form method=\"POST\" action=\"$::dmailTivoURL/handle_multi_events/\" name=\"todoform\" target=\"_blank\">"
		writehtml "<table class=\"section\">"

		if { [catch { set itemsFound [ gettodo $::dmailHoursahead ] } fid] } {
			set itemsFound -1
			dmdebug 1 "ERROR IN GETTODO MODULE >>$fid<<"
		}
		dmdebug 2 "Items Found >>$itemsFound<<"

		if { $itemsFound == 0 } {
			writehtml "<tr><td class=\"programme\">None</td></tr>"
		}
		if { $itemsFound == -1 } {
			writehtml "</table><table><tr><td class=\"programme\">Error When Generating To Do List - See Log - $fid</td></tr>"
			dmdebug 1 "---------------------------------------------------------------- \n $errorInfo \n ----------------------------------------------------------------" false
		}

		writehtml "</table>"
		writehtml "</form>"
		
		
	# ###################################################################################################
	# # NOW SHOWING LIST CODE
	} elseif { $index == $::dmailShowing } {

		dmdebug 2 "generating now showing list"

		writehtml "<br><span class=\"SectionHeading\">Now Showing</span><br>"
		writehtml "<table><tr><td class=\"header\">"
		if {$::nowshowingdaylimit != 0} {
			writehtml "The following recordings are currently available on this TiVo. This list only includes those p${::program_word}s recorded in the last $::nowshowingdaylimit days:"
		} else {
			writehtml "The following recordings are currently available on this TiVo: "
		}
		writehtml "</td></tr></table>"
		writehtml "<form method=\"POST\" action=\"$::dmailTivoURL/handle_multi_events/\" name=\"nowshowingform\" target=\"_blank\">"
		writehtml "<table class=\"section\">"

		if { [catch { set itemsFound [ getnowshowing $::nowshowingdaylimit ] } fid] } {
			set itemsFound -1
			dmdebug 1 "ERROR IN NOW SHOWING MODULE >>$fid<<"
		}  
		dmdebug 2 "Items Found >>$itemsFound<<"

		if { $itemsFound == 0 } {
			writehtml "<tr><td class=\"programme\">None</td></tr>"
		}	
		if { $itemsFound == -1 } {
			writehtml "</table><table><tr><td class=\"programme\">Error When Generating Now Showing List - See Log - $fid</td></tr>"
			dmdebug 1 "---------------------------------------------------------------- \n $errorInfo \n ----------------------------------------------------------------" false
		}

		writehtml "</table>"
		writehtml "</form>"
	
	
	# ###################################################################################################
	# # CONFLICTS LIST CODE
	#
	} elseif { $index == $::dmailConflicts } {
	
		dmdebug 2 "generating conflicts list"
        
		writehtml "<br><span class=\"SectionHeading\">Conflicts</span><br>"
		writehtml "<table><tr><td class=\"header\">"
		writehtml "The following unresolvable conflicts occur in the next $::dmailHoursahead hours. "
		writehtml "TiVo has determined that these p${::program_word}s that you have selected to be recorded via season-pass or wishlist <b>will not be recorded</b>:"
		writehtml "</td></tr></table>"
		writehtml "<br><table class=\"section\">"
		
		if { [catch { set itemsFound [ getconflicts $::dmailHoursahead ] } fid] } {
			set itemsFound -1
			dmdebug 1 "ERROR IN CONFLICTS MODULE >>$fid<<"
		}  
		dmdebug 2 "Items Found >>$itemsFound<<"
			
		if { $itemsFound == 0 } {
			writehtml "<tr><td class=\"programme\">None</td></tr>"
		}
		if { $itemsFound == -1 } {
			writehtml "</table><table><tr><td class=\"programme\">Error When Generating Conflicts List - See Log - $fid</td></tr>"
			dmdebug 1 "---------------------------------------------------------------- \n $errorInfo \n ----------------------------------------------------------------" false
		}

		if {$::dmailLinksOn} {
			if { $itemsFound > 0 } {
				if { $::confresInstalled } {
					if { $::confresUseForm } {
						writehtml "<tr><td class=justfont>"
						writehtml "<b>Conflict Resolver</b>"
						writehtml "<br>"
						writehtml "<TABLE border=\"0\" cellspacing=\"0\" cellpadding=\"0\" >"
						writehtml "<FORM METHOD=\"GET\" ACTION=\"$::dmailTivoURL/findconfs\" target=\"_blank\">"
						writehtml "<TR><TD class=\"justfont\">Alternate TiVo (name or IP Address): </TD>"
						writehtml "<TD class=\"justfont\"><INPUT TYPE=\"TEXT\" NAME=\"tivoname\" SIZE=\"20\" VALUE=\"$::confresDefaultOtherTiVo\"></TD></TR>"
						writehtml "<TR><TD class=\"justfont\">Number of days ahead to look: </TD>"
						writehtml "<TD class=\"justfont\"><INPUT TYPE=\"TEXT\" NAME=\"daysahead\" SIZE=\"20\" VALUE=\"$::confresDefaultDaysAhead\"></TD></TR>"
						writehtml "<TR><TD class=\"justfont\">&nbsp;</TD><TD><INPUT TYPE=\"submit\" NAME=\"Save\" VALUE=\"Next\"></TD></TR>"
						writehtml "</FORM>"
						writehtml "</TABLE>"
					} else {
						writehtml "<tr><td class=\"justfont\" style=\"padding:5px 0 5px 0\">"
						writehtml "<a align=\"center\" href=\"$::dmailTivoURL/findconfs?tivoname=$::confresDefaultOtherTiVo&daysahead=$::confresDefaultDaysAhead&Save=Next\" target=\"_blank\" title=\"Click here to resolve conflicts.\">"
						writehtml "Click here to resolve these conflicts</a> with the designated alternate TiVo (${::confresDefaultOtherTiVo})."
					}
					writehtml "</td></tr>"
				}
			}
		}
		writehtml "</table>"
		
		
	# ###################################################################################################
	# # EXPIRED LIST CODE
	#
	} elseif { $index == $::dmailExpired } {

		dmdebug 2 "generating expiry list"
		writehtml "<br><span class=\"SectionHeading\">E${::expiry_word}</span><br>"
		writehtml "<table><tr><td class=\"header\">"
		writehtml "The following recordings are due to expire in the next $::dmailHoursahead hours: "
		writehtml "</td></tr></table>"
		writehtml "<br><table class=\"section\">"

		if { [catch { set itemsFound [ getnpexpire $::dmailHoursahead ] } fid] } {
			set itemsFound -1
			dmdebug 1 "ERROR IN EXPIRY MODULE >>$fid<<"
		}  
		dmdebug 2 "Items Found >>$itemsFound<<"

		if { $itemsFound == 0 } {
			writehtml "<tr><td class=\"programme\">None</td></tr>"
		}	
		if { $itemsFound == -1 } {
			writehtml "</table><table><tr><td class=\"programme\">Error When Generating Expired List - See Log - $fid</td></tr>"
			dmdebug 1 "---------------------------------------------------------------- \n $errorInfo \n ----------------------------------------------------------------" false
		}

		writehtml "</table>"
		
		
	# ###################################################################################################
	# # STATUS LIST CODE
	#
	} elseif { $index == $::dmailTivoStatus } {

		dmdebug 2 "generating status list"
		writehtml "<br><span class=\"SectionHeading\">Status</span><br>"
		writehtml "<table><tr><td class=\"header\">"
		if {$::yankspeak} {
			writehtml "Here is the status of your TiVo as of <i>[clock format [expr [clock seconds] + $::tzoffset] -format {%A, %B %d %I:%M:%S%p}]</i>: "
		} else {
			writehtml "Here is the status of your TiVo as of <i>[clock format [clock seconds] -format {%a, %d %b %H:%M} -gmt true]</i>: "
		}
		writehtml "</td></tr></table>"
		writehtml "<br><table class=\"section\">"

		if { [catch { gettempuptime } fid] } {
			set itemsFound -1
			dmdebug 1 "ERROR IN STATUS MODULE >>$fid<<"
		} else {
			dmdebug 3 "GetTempUpTime module completed ok"
		}
		
		if { $itemsFound == -1 } {
			writehtml "</table><table><tr><td class=\"programme\">Error When Generating Status List - See Log - $fid</td></tr>"
			dmdebug 1 "---------------------------------------------------------------- \n $errorInfo \n ----------------------------------------------------------------" false
		}
		writehtml "</table>"
		
		
	# ###################################################################################################
	# # RECORDING HISTORY LIST CODE
	#
	} elseif { $index == $::dmailRecHistory } {
	
		dmdebug 2 "generating record history list"
		writehtml "<br><span class=\"SectionHeading\">Rejected Recordings</span><br>"
		writehtml "<table><tr><td class=\"header\">"
		if {$::rejectedrecreq == 0} {
			writehtml "The following p${::program_word}s either were not recorded in the last $::dmailHoursahead hours or will not be recorded within the next $::dmailHoursahead hours : "
		} elseif {$::rejectedrecreq == -1} {
			writehtml "The following p${::program_word}s were not recorded in the last $::dmailHoursahead hours : "
		} elseif {$::rejectedrecreq == 1} {
			writehtml "The following p${::program_word}s will not be recorded within the next $::dmailHoursahead hours : "
		} else {
			writehtml "No rejected recordings list information available."
		}
		writehtml "</td></tr></table>"
		writehtml "<br><table class=\"section\">"

		if { [catch { set itemsFound [ getrechistory $::dmailHoursahead $::rejectedrecreq ] } fid] } {
			set itemsFound -1
			dmdebug 1 "ERROR IN RECORD HISTORY MODULE >>$fid<<"
		} 	
		dmdebug 2 "Items Found >>$itemsFound<<"

		if { $itemsFound == 0 } {
			writehtml "<tr><td class=\"programme\">None</td></tr>"
		}
		if { $itemsFound == -1 } {
			writehtml "</table><table><tr><td class=\"programme\">Error When Generating Rejected Recording List - See Log - $fid</td></tr>"
			dmdebug 1 "---------------------------------------------------------------- \n $errorInfo \n ----------------------------------------------------------------" false
		}
		writehtml "</table>"
	
	
	# ###################################################################################################
	#
	} else {
		dmdebug 2 "section $index not specified in config file - skipping"
	}
	
	dmdebug 3 "handle section module completed ok"
	
}





# #######################################################################################################
# #######################################################################################################
# # MAIN CODE

# initial logging text
puts "\n\n------welcome to the jazzed dailymail program - v$software_version ($software_release_date) - starting"



# #######################################################################################################
# 1. configuration & setup

# variable to specify whether the memory cataloging function should run at the end of the module.
set memcat false

# some default setup variables for earlier version code users - will be overriden by config file if current version used
set externalipsetting false
set oursiteport ""
set oursitevirtualpath ""
set oursitesecure false
set externalsiteip ""
set externalsitestartmap ""
set externalsiteendmap ""
set externallogging false
set sortnowshowing ""
set nowshowingrepeatfreq 15
set multicolourbar false
set sortnowshowingascdesc "ASC"
set nowshowingdaylimit 0

# setup alert option variables
set alertstatus false
set alertreasons ""

# attach configuration file
set pathname [file dirname [info script]]
source $pathname/dailymail.cfg

# attach procedures used
source $pathname/dailymail_jazz_libs.itcl
source $pathname/dailymail_jazz_procs.itcl

if {$yankspeak} {
	set datetimeformat {%A, %B %d %I:%M%p}
	set dateformat {%A, %B %d}
	set shortdatetimeformat {%A %I:%M%p}
	set longdatetimeformat {%A, %B %d %Y, %I:%M%p}
	set timeformat {%I:%M%p}
	set dayformat {%A}
} else {
	set datetimeformat {%a, %d %b %H:%M}
	set dateformat {%a, %d %b}
	set shortdatetimeformat {%a %H:%M}
	set longdatetimeformat {%a, %d %b %Y, %H:%M}
	set timeformat {%H:%M}
	set dayformat {%a}
}

# logging text regarding language and start time
puts "------the debug level is set at >>$debuglvl<<"
if {$yankspeak} { puts "------generated content presented in American English for your reading pleasure!" }
puts "------started at [clock format [clock seconds] -format $longdatetimeformat -gmt true] (debugging times specified in gmt)\n"

# validate that pool size is large enough to run script - else cop out
if {[pool pool0 size] <= 1622016} {
	puts "Use the 'dailymail_jazz' script to start the dailymail_jazz module!"
	puts "\n------completed the jazzed dailymail program - v$software_version ($software_release_date) - thanks for your custom"
	puts "------completed at [clock format [clock seconds] -format {%H:%M:%S} -gmt true]\n\n"
	exit
}

# setup the unique id required for the cid when using embedded images
set uniquecontentid [clock seconds]

# setup the image file name array required for image embedding option
set imagenamelist "$tivologofilename $tivofilename $tivobrokenfilename $phonefilename $phonebrokenfilename $diskfilename $diskbrokenfilename $diskfullfilename $diskfullbrokenfilename $endpadonfilename $endpadofffilename $mailfilename $reddotonfilename $reddotofffilename $spacegraphicstart $spacegraphiclow $spacegraphicmid $spacegraphichigh $spacegraphicempty $spacegraphicend $explicitonethumbupgraphic $explicittwothumbupgraphic $explicitthreethumbupgraphic $explicitonethumbdowngraphic $explicittwothumbdowngraphic $explicitthreethumbdowngraphic $impliedonethumbupgraphic $impliedtwothumbupgraphic $impliedthreethumbupgraphic $impliedonethumbdowngraphic $impliedtwothumbdowngraphic $impliedthreethumbdowngraphic $predictedonethumbupgraphic $predictedtwothumbupgraphic $predictedthreethumbupgraphic $predictedonethumbdowngraphic $predictedtwothumbdowngraphic $predictedthreethumbdowngraphic $unsetthumbsgraphic $endpadrecordingunlikelygraphic $emergencytempfilename $optionalmessagefilename $mutideletefilename $multiupdatethumbsfilename $multiupdatethumbsblankboxfilename $trackerblockfilename $spacegraphicextra $suggestiongraphic $exclaimgraphic $startpaddinggraphic $endpaddinggraphic $spaceaddedbackfilename $deleteforroomfilename $flashingexclaimgraphic $newepisodegraphic $seasonpassgraphic $spacebarinfographic $unblockmegraphic $blockmegraphic"

set imagenamesused {}

set thumbimages "{} {} $::explicittwothumbupgraphic {} $::explicitonethumbdowngraphic {} $::explicittwothumbdowngraphic $::explicitthreethumbupgraphic {} $::explicitthreethumbdowngraphic {} {} $::explicitonethumbupgraphic {} $::predictedthreethumbdowngraphic $::predictedtwothumbdowngraphic $::predictedonethumbdowngraphic $::predictedonethumbupgraphic $::predictedtwothumbupgraphic $::predictedthreethumbupgraphic {} {} $::impliedthreethumbdowngraphic $::impliedtwothumbdowngraphic $::impliedonethumbdowngraphic $::impliedonethumbupgraphic $::impliedtwothumbupgraphic $::impliedthreethumbupgraphic"
 
 
	
# move the list seperator offset from hours to seconds
set listseperatoroffset [expr $listseperatoroffset * 3600]

# sort out table width in px
set programbarwidthpx [expr round($programbarwidthpercent * 7.5) ]

# TiVo database handle
set db [dbopen]

# Mail module path deiffers between TW/TWP1.3 & TWP2.1 (assume 2.1 by default)
if {$::tivowebplususer && (![info exists ::tivowebplusversion] || $::tivowebplusversion == 2.1)} {
 set linkmail "mail/List"
} else {
 set linkmail "listmail"
}


#needed to fool TiVoWeb modules that they are being called by TiVoWeb :)
#
global reload
global dlc
set reload 1
set dlc 1
set channeltablefsid {}

setpri fifo 1
catch {EnableTransactionHoldoff true}


# get the script's arguments
#
set recipients ""
set hoursahead -1

dmdebug 1 "$argc argument(s) found"

for {set i 0} {$i < [min $argc 4]} {incr i} {
	set argument  [lindex $argv $i]
	dmdebug 2 "argument $i passed in : >>$argument<<"
	if {$argument != ""} {
		if {[regexp {.*@.*\..*} $argument]}	{											;# rough check for e-mail address
			dmdebug 2 "a new recipient email found in argument $i"
			lappend recipients $argument
		} elseif {[regexp {^[0-9]*$} $argument]}	{							;# 'hours' parameter
			dmdebug 2 "a new hours ahead value found in argument $i"
			set hoursahead $argument
		} else {
			puts "------unrecognised parameter >>$argument<<"
		}
	}
}

# validate the arguments
#
if {[llength $recipients] > 3} {
	dmdebug 2 "only 3 email recipients allowed - discarding number 4"
}

if {[llength $recipients] > 0} {
	# if any emails are sent via arguments - then all default ones are cleared
	# if no emails are sent via argument - then original emails stand
	for {set i 0} {$i < 3} {incr i} {
		set recipient[expr $i>0?[expr $i+1]:""] [lindex $recipients $i]
	}
	if {$recipient != ""} {dmdebug  0 "recipient email value is now : >>$recipient<<"}
	if {$recipient2 != ""} {dmdebug 0 "recipient2 email value is now : >>$recipient2<<" }
	if {$recipient3 != ""} {dmdebug  0 "recipient3 email value is now : >>$recipient3<<" }
}

# store the config value (see footer section
set confighoursahead $dmailHoursahead

if {$hoursahead != -1} {
	if {$hoursahead <= 0  || $hoursahead > 500} {
		dmdebug 1 "hours ahead value invalid (range 1 to 500)"
		dmdebug 1 "using original hours defined in configuration file!"
	} else {
		set dmailHoursahead $hoursahead
		dmdebug 1 "hours ahead value is now : >>$dmailHoursahead<<"
	}
}

if {$dmailHoursahead <= 0  || $dmailHoursahead > 500} {
	puts "------hours ahead value invalid (range 1 to 500) >>$dmailHoursahead<<"
}

catch { unset recipients hoursahead }
dmdebug 1 "argument handling complete"

	
dmdebug 1 "CONFIGURATION & SETUP - COMPLETED"
# END 1.
# #######################################################################################################


# #######################################################################################################
# 2. Handle freepages memory adjustment
	
if {$freepagesadjustment} {
	dmdebug 1 "MEMORY FREEPAGES ADJUSTMENT - START"
	if { [ catch { 
		set opt [open /proc/sys/vm/freepages "w"] 
		puts $opt "128 192 256"
		close $opt
		after 2000
		} ] } {
		dmdebug 1 "memory freepages adjustment caused ERROR"
	} else {
		dmdebug 1 "successfully completed memory freepages adjustment"
	}
	catch {	unset opt }
	dmdebug 1 "MEMORY FREEPAGES ADJUSTMENT - COMPLETED"
}

# END 2.
# #######################################################################################################
	
	
# #######################################################################################################
# 3. Set up ip information

dmdebug 1 "IP EXTERNAL SETUP SECTION - STARTED"

if 1 {
	dmdebug 2 "getting local ip address"
	catch {
		# compute this TiVo's IP address
		# search the configured ips and take the first non-localhost one
		set ip_address_strs [split [string trim [exec ifconfig | grep "inet addr" ]] "\n"]
		foreach ip_address_str $ip_address_strs {
			if {[regexp {127.0.0.1} $ip_address_str]} { continue }
			regexp {inet addr:([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)} $ip_address_str junk ouripaddress
			break
		}
		dmdebug 2 "our ip : >>$ouripaddress<<"
		# adjust cfg params if user hasn't set them!
		if { $tivowebplususer && ![regexp {^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$} $tivoip] } {
			set tivoip $ouripaddress
		}
		if { ![regexp {^http://[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$} $dmailTivoURL] } {
			set dmailTivoURL "http://$ouripaddress"
		}
	}
}

if {$externalipsetting} {
	dmdebug 2 "getting ip address"
	catch {
		# attach ip utilities script
		source $pathname/ip-utilities.tcl 
		
		set ouripaddress [get_external_IP $externalsiteip $externalsitestartmap $externalsiteendmap]
		dmdebug 2 "old tivo url : >>$dmailTivoURL<<"
		dmdebug 2 "external ip : >>$ouripaddress<<"
		if {$oursitesecure} {
			set dmailTivoURL "https://${ouripaddress}"
		} else {
			set dmailTivoURL "http://${ouripaddress}"
		}
		if {$oursiteport != ""} {
			set dmailTivoURL "${dmailTivoURL}:${oursiteport}"
		}
		if {$oursitevirtualpath != ""} {
			set dmailTivoURL "${dmailTivoURL}/${oursitevirtualpath}"
		}
		dmdebug 2 "new tivo url : >>$dmailTivoURL<<"
	}
}

if {![info exists ::altepgemailservice] || !$::altepgemailservice} {

	# if ip not provided then attempt to get mailserver ip using fake dns
	dmdebug 2 "checking mailserver ip format"
	catch {
		if {![regexp {^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$} $mailserver]} {
			dmdebug 3 "mailserver not an ip - attempt to get ip via fake dns"
			dmdebug 3 "mailserver url : >>$mailserver<<"
			set mailserver [fake_dns $mailserver]
			dmdebug 3 "mailserver ip : >>$mailserver<<"
		} 
	}
}

dmdebug 1 "IP EXTERNAL SETUP SECTION - COMPLETED"
		
# END 3.
# #######################################################################################################


# #######################################################################################################
# 4. Handle fakecall module
	
if {$runfakecallmodule} {
	dmdebug 1 "FAKECALL FUNCTIONALITY - START"
	if { [ catch { set execoutput [exec $fakecallexec] } ] } {
		dmdebug 1 "fakecall module caused an ERROR"
	} else {
		dmdebug 1 "successfully completed running fakecall module"
	}
	catch { unset execoutput }
	dmdebug 1 "FAKECALL FUNCTIONALITY - COMPLETED"
}
	
# END 4.
# #######################################################################################################

	
# #######################################################################################################
# 10. Message handling

if 1 {
	dmdebug 1 "OPTIONAL MESSAGE FILE READING - START."
	
	set optionalmsg {}

	if { [file exists $msgfilename] } {
		dmdebug 2 "message file found"
		catch {
			dmdebug 2 "fsize : >>[file size "$msgfilename"]<<"
			set msgfile [open "$msgfilename" "r"]
			lappend optionalmsg [read $msgfile]
			dmdebug 2 "lines : \n>>\n[strim $optionalmsg]\n<<"
			close $msgfile
		}
		catch { file delete -force $msgfilename }
	} else {
		dmdebug 2  "no message file found"
	}
		
	catch { unset msgfilename msgfile }
	dmdebug 1 "OPTIONAL MESSAGE FILE READING - COMPLETED."
}

# END 10.
# #######################################################################################################


# #######################################################################################################
# 20. Version setting code

if 1 {
	dmdebug 1 "VERSION IDENTIFICATION - START."
	
	set dtivo [MfsFileExists /ApgBoot]
	RetryTransaction {
		if {[catch {
			if { [info exists env(SerialNumber)] && [string trim $env(SerialNumber) "0"] != "" && [string length $env(SerialNumber)] == 15 } {
				set sn $env(SerialNumber)
			} else {
				set sn [exec /tvbin/crypto -gsn]
			}
			set model [string index $sn 0]
			set manuf [string index $sn 1]
			set opt [string index $sn 2]
			switch $manuf {
				0		{ set manufstring "Philips" }
				1		{ set manufstring "Sony"  }
				2		{ set manufstring "Thompson/RCA" }
				3		{ set manufstring "Hughes/AT&T" }
				4		{ set manufstring "TiVo" }
				5		{ set manufstring "Hughes"  }
				6		{ set manufstring "Toshiba" }
				7		{ set manufstring "Pioneer" }
				8		{ set manufstring "Samsung" }
				default		{ set manufstring "Other"  }
			}
			switch $opt {
				0		{ set optstring "StandAlone" }
				1		{ set optstring "DirecTiVo"  }
				2		{ set optstring "StandAlone" }
				3		{ set optstring "UK/PAL" }
				4		{ set optstring "DVD" }
				5		{ set optstring "DVD Writer"  }
				default		{ set optstring "Other"  }
			}		
			dmdebug 2 "TiVo Model : >>$model<<."
			dmdebug 2 "TiVo Manufacturer : >>$manufstring<<."
			dmdebug 2 "TiVo Type : >>$optstring<<."
		}]} {
			set sn -1
			set model "Unknown"
			set manufstring "Unknown"
			set optstring "Unknown"
			set manuf -1
			set opt -1
			dmdebug 1 "ERROR - Cannot find manufacturer, model or serial information."
		}
		set swsystem [db $db open /SwSystem/ACTIVE]
		set tivoswversion [dbobj $swsystem get Name]
		set setup [db $db open /Setup]
		dmdebug 2 "TiVo Software version : >>$tivoswversion<<."
		set version [string index $tivoswversion 0]
		set suffix [string index $tivoswversion [expr [string length $tivoswversion] - 5]]
		set uktivo [expr [string index $tivoswversion [expr [string length $tivoswversion] - 1]] == 3]
		if { $suffix == 2 } {
			set series1 0
		} else {
			set series1 1
		}
		if { $series1 } {
			dmdebug 2 "TiVo : >>Series 1<<."
		} else {
			dmdebug 2 "TiVo : >>Series 2<<."
		}
		if {$version >= 3} {
			set version3 1
			set lconfig  [db $db open /State/LocationConfig]
			set setup [db $db open /State/ServiceConfig]
			if { $version >=4 } {
				set setuptz [dbobj $lconfig get TimeZoneOffset]
			} else {
				set setuptz [dbobj $lconfig get TimeZoneOld]
			}
			if { $::version >=5 } {
				set daylightsavings ""
			} else {
				set daylightsavings [dbobj $lconfig get DaylightSavingsPolicy]
			}
			set avconfig [db $db open /State/AvConfig]
		} else {
			set version3 0
			set setuptz [dbobj $setup get TimeZone]
			set daylightsavings [dbobj $setup get DaylightSavingsPolicy]
			set avconfig [db $db open /Setup]
		}
		dmdebug 3 "setuptz identified : >>$setuptz<<."
		dmdebug 3 "daylightsavings identified : >>$daylightsavings<<."
		
		set tivoswversion [string range $tivoswversion 0 2]
		set tivoswversionloc [string first "-" $tivoswversion]
		if {$tivoswversionloc != -1} {
			set tivoswversion [string range $tivoswversion 0 $tivoswversionloc]
		}
		dmdebug 2 "cleaned Up TiVo Software version : >>$tivoswversion<<."
		
		set num_tuners [dbobj $avconfig get TunerCount]
		if {$num_tuners == ""} {
			set num_tuners 1
		}
		if { $num_tuners > 1 } {
			set endpadtodoliston false
			dmdebug 1 "more than 1 tuner found - 'To Do' Endpad functionality set OFF : >>$num_tuners<<."
		} else {
			set endpadtodoliston true
			dmdebug 1 "just 1 tuner found - 'To Do' Endpad functionality set ON : >>$num_tuners<<."
		}
	}
	
	dmdebug 2 "is TiVo UKTiVo? : >>$uktivo<<."
	dmdebug 2 "is TiVo Direct TiVo : >>$dtivo<<."
	
	catch { unset avconfig num_tuners }
	
	set ::tcid $sn
	if {$::tivoName == ""} { set tivoName "SerNo.$tcid" }
	
	dmdebug 1 "VERSION IDENTIFICATION - COMPLETED."
}

# END 20.
# #######################################################################################################



# #######################################################################################################
# 22. Calling TiVoWeb procedures

if 1 {
	dmdebug 1 "INITIALISING SEARCH TABLES  - START."
	
	# From TiVoWeb's index.itcl
	
	if {$version3} {
		 set guideindexdir "/GuideIndexV2"
	} else {
		 set guideindexdir "/GuideIndex"
	}

	if {$version3} {
		 set seasonpassdir "/SeasonPass/User"
	} else {
		 set seasonpassdir "/SeasonPass"
	}

	init_logoindex
	init_channelindex
	init_genreindex

	dmdebug 1 "INITIALISING SEARCH TABLES  - COMPLETED."
}

# END 22.
# #######################################################################################################




# #######################################################################################################
# 25. Set timezone offset
	
if 1 {
	dmdebug 1 "ESTABLISHING TIMEZONE OFFSET - START"
	
	dmdebug 3 "initial setuptz value identified >>$setuptz<<"
	dmdebug 3 "initial daylightsavings value identified >>$daylightsavings<<"		
	set override false
	
	## if setuptz already set, then we may need to turn it from seconds to hours
	if { $setuptz != "" } {
		dmdebug 3 "setuptz identified as blank"		
		if {$setuptz < -24} {
			dmdebug 3 "setuptz converted from seconds to hours (negative)"
			## currently in seconds
			set setuptz [expr $setuptz / 3600] 
		}
		if {$setuptz > 24} {
			dmdebug 3 "setuptz converted from seconds to hours (positive)"
			## currently in seconds
			set setuptz [expr $setuptz / 3600] 
		}
	}
	dmdebug 3 "current setuptz value >>$setuptz<<"
		
	## handle time offset in evironment variable if present
	if { [catch {set tz [set env(TZ_GMT_OFFSET)]} ] } {
		## NO ENVIRONMENT VARIABLE FOUND
		dmdebug 3 "external TZ_GMT_OFFSET environment variable has not been set - No override occurred!"
	} else {
		## FOUND ENVIRONMENT VARIABLE
		dmdebug 3 "external TZ_GMT_OFFSET environment variable has been set - Overrides default!"
		if {$tz != ""} {
			## ENVIRONMENT VARIABLE IS NOT BLANK - USE IT FOR TIME OFFSET SETTING
			set setuptz $tz
			dmdebug 3 "timezone offset after change : >>$setuptz<<."
			set override true
		}
	}
	dmdebug 3 "current setuptz value >>$setuptz<<"
	
	## handle time offset in configuration variable if set
	if {$manualtzoffset != ""} {
		## CONFIG VARIABLE SET - USE IT FOR TIME OFFSET SETTING
		set setuptz $manualtzoffset
		dmdebug 3 "manual timezone offset set - Override occurred : >>$setuptz<<."
		set override true
	}
  	dmdebug 3 "daylightsavings : >>$daylightsavings<<."
	if {$setuptz == ""} {
		dmdebug 3 "default setuptz to 0"
 		set setuptz 0
	}
	if {$daylightsavings == ""} {
		dmdebug 3 "default daylightsavings to 2"
  	set daylightsavings 2
	}
	
	dmdebug 3 "Timezone offset after additional change : >>$setuptz<<."
	dmdebug 3 "Daylightsavings after possible change: >>$daylightsavings<<."
	
	set tzoffset [dmget_tzoffset $setuptz $daylightsavings $override]
	dmdebug 3 "tzoffset value before finalization >>$tzoffset<<"
	
	set tzoffset [expr round(([clock scan [lreplace [exec date -u] 4 4]] - [clock seconds] + $tzoffset)/60.0)*60]
	dmdebug 2 "identified time zone offset : >>$tzoffset<<."
	
	catch { unset setuptz daylightsavings }
	dmdebug 1 "ESTABLISHING TIMEZONE OFFSET - COMPLETED"
}

# END 25.
# #######################################################################################################


# #######################################################################################################
# 40. Identification of default recording quality for space calculations.
	
if 1 {
	dmdebug 1 "GETTING DEFAULT RECORDING QUALITY - START."
	
	if {$version >= 3} {
		dmdebug 3 "software version is 3 or greater."
		RetryTransaction {
			set avconfig [db $db open /State/AvConfig]
			set defrecquality [defaultval 100 [dbobj $avconfig get RecordQuality]]
			catch { unset avconfig }
		}
	} else {
		dmdebug 3 "software version is less than 3."
		RetryTransaction {
			set setup [db $db open /Setup]
			set defrecquality [defaultval 100 [dbobj $setup get RecordQuality]]
			catch { unset setup  }
	    }
	}
	dmdebug 2 "default recording quality : >>$defrecquality<<."
	
	catch { unset avconfig setup }
	dmdebug 1 "GETTING DEFAULT RECORDING QUALITY - COMPLETED."
}
	
# END 40.
# #######################################################################################################


# #######################################################################################################
# 50. Other generic variable setting.
	
if 1 {
	dmdebug 1 "SETTING MFS DIRECTORY - START."	
	
	if {$version >= 3} {
	    set nowshowingdir "/Recording/NowShowingByClassic"
	} else {
	    set nowshowingdir "/Recording/NowShowing"
	}
	dmdebug 2 "setting 'now showing' directory >>$nowshowingdir<<"
	
	dmdebug 1 "SETTING MFS DIRECTORY - COMPLETED."
}

# END 50.
# #######################################################################################################


# #######################################################################################################
# 52. Global Blocklist setup
	
if {$::globalblockreq} {
	dmdebug 1 "LOADING GLOBAL BLOCKLISTS - START."	
		
	# TODO - these should be added to the config file
	if {![info exists ::blocklistdir]} { set ::blocklistdir "/var/hack/etc" }
	if {![info exists ::blocklistfile]} { set ::blocklistfile "$::blocklistdir/blocklist.txt" }
	if {![info exists ::blockpattfile]} { set ::blockpattfile "$::blocklistdir/blockpatt.txt" }

	set ::blocklist [gbl_load $::blocklistfile]
	set ::blockpatt [gbl_load $::blockpattfile]

	dmdebug 1 "LOADING GLOBAL BLOCKLISTS - END."	
}

# END 52.
# #######################################################################################################


# #######################################################################################################
# 54. Tracker Blocklist setup
	
if {$::trackerblockreq} {
	dmdebug 1 "LOADING TRACKER BLOCKLISTS - START."	
		
	tracker_init
	
	dmdebug 3 "- lastupdate >>$lastupdate<<"
	dmdebug 3 "- blockwatchedID >>$blockwatchedID<<"
	dmdebug 3 "- blockunwatchedID >>$blockunwatchedID<<"
	dmdebug 3 "- blockwatchedTD >>$blockwatchedTD<<"
	dmdebug 3 "- blockunwatchedTD >>$blockunwatchedTD<<"
	dmdebug 3 "- ignoregenerics >>$ignoregenerics<<"
	
	dmtracker_load
	
	dmdebug 1 "LOADING TRACKER BLOCKLISTS - END."	
}

# END 54.
# #######################################################################################################



# #######################################################################################################
# 60. external logging
  
if {$externallogging} {
	dmdebug 1 "EXTERNAL LOGGING - START."
	catch { 
		set detailget [send_details $recipient $tivoName [clock format [expr [clock seconds] + $tzoffset] -format {%a, %d %b %H:%M} -gmt true] $software_version $software_release_date $debuglvl $dmailHoursahead $version $model $manufstring $optstring $series1 $uktivo]
	}
	dmdebug 1 "EXTERNAL LOGGING - COMPLETED."
}

# END 60.
# #######################################################################################################
	
	
# unset some variables which are no longer required
catch { unset manuf model optstring sn manufstring tivoswversionloc swsystem suffix }


# #######################################################################################################
# EMAIL ASSEMBLY
#
	
set message {}
dmdebug 1 "ASSEMBLING EMAIL:"


# #######################################################################################################
# 70. Create Email heading
	
if 1 {
	dmdebug 1 "CREATING EMAIL HEADINGS - START"
	
	set htmlfilefd 0
	writehtml "<table width=\"580\" border=\"0\"> <tr>"
	if {$graphicson} {
		writehtml "<td width=\"72\"><img src=\"[imageref $tivologofilename]\" alt=\"TiVo Logo\"></td>"
	}
	writehtml "<td width=\"508\" class=PageHeading>"
	if {$tivoName==""} {
		set tempTiVoName ""
	} else {
		set tempTiVoName "$tivoName "
	}
	if {$yankspeak} {
		writehtml "${tempTiVoName}TiVo Status: [clock format [expr [clock seconds] + $tzoffset] -format {%A, %B %d %I:%M:%S%p}]"
	} else {
		writehtml "${tempTiVoName}TiVo Status: [clock format [expr [clock seconds] + $tzoffset] -format {%a, %d %b %H:%M} -gmt true]"
	}
	if {$graphicson} {} else { writehtml "<br><br>" }
	writehtml "</td></tr></table>"

	dmdebug 1 "CREATING EMAIL HEADINGS - COMPLETED"
}

# END 70.
# #######################################################################################################


# #######################################################################################################
# 80. Create optional message text.

if {$optionalmsg != {}} {
	dmdebug 1 "CREATING OPTIONAL MESSAGE - START"
	
	if { [ lindex $optionalmsg 0 ] != ""} {
		writehtml "<table width=\"450\" border=\"0\"class=Indented><tr>"
		if {$graphicson} {
			writehtml "<td width=\"80\" class=Message><center><img src=\"[imageref $optionalmessagefilename]\" border=\"0\" alt=\"Optional Message\"></center></td><td width=\"370\" class=Message>"
		} else {
			writehtml "<td width=\"450\" class=Message>"
		}
		writehtml [nl2br [strim $optionalmsg]]
			if {$graphicson} {} else { writehtml "<br><br>" }
			writehtml "</td></tr></table>"					
	}
	
	catch { unset optionalmsg } 
	dmdebug 1 "CREATING OPTIONAL MESSAGE - COMPLETED"
}

# END 80.
# #######################################################################################################


# #######################################################################################################
# 90. TiVo Status / Reboot Info

if 1 {
	dmdebug 1 "TiVo STATUS / REBOOT INFO - START"
	
	writehtml "<table width=\"[expr $programbarwidthpx + 250]\" border=\"0\" class=indented>"
	set uptime [lindex [exec cat /proc/uptime] 0]
	if {$bulletinverbose || $uptime < 86400} {
		if {$graphicson} {
			if {$uptime < 86400} {
				set ourimagefilename $tivobrokenfilename
			} else {
				set ourimagefilename $tivofilename
			}
			if {$dmailLinksOn} {
				writehtml "<tr><td width=\"80\"><center><a href=\"$dmailTivoURL/info/\" target=\"_blank\" ><img src=\"[imageref $ourimagefilename]\" border=\"0\" title=\"Click&nbsp;here&nbsp;to&nbsp;view&nbsp;more&nbsp;TiVo&nbsp;Information.\" alt=\"TiVo Info\"></a></center></td>"
			} else {
				writehtml "<tr><td width=\"80\"><center><img src=\"[imageref $ourimagefilename]\" border=\"0\" alt=\"TiVo Info\"></center></td>"
			}
			writehtml "<td width=\"[expr $programbarwidthpx + 170]\""
		} else {
			writehtml "<tr><td width=\"[expr $programbarwidthpx + 250]\""
		}
		if {$uptime < 86400} {
			writehtml " class=Warning>WARNING: The TiVo has rebooted within the last 24 hours."
			set alertstatus true
			append alertreasons "Rebooted - "
		} else {
			writehtml " class=Normal>The TiVo has not rebooted within the last 24 hours."
		}
		if {$graphicson} {} else { writehtml "<br><br>" }
		writehtml "</td></tr></table>"	
	}
	
	catch { unset uptime }
	dmdebug 1 "TiVo STATUS / REBOOT INFO - COMPLETED"
}
	
# END 90.
# #######################################################################################################


# #######################################################################################################
# 100. Emergency Temp Guage Section

if 1 {
	dmdebug 1 "CHECKING FOR EMERGENCY TEMP READING - START"
	
	writehtml "<table width=\"[expr $programbarwidthpx + 250]\" border=\"0\" class=indented>"
	set cur_temp [exec cat /proc/therm]
	dmdebug 3 "Temperature check run successfully >>$cur_temp<<"
	set digit_tempC [expr [string trim $cur_temp 'C']]
	set cur_tempF [expr 1.8*[string trim $cur_temp 'C']+32]F
	if {$digit_tempC > $hightempcentigrade && $hightempcentigrade != 0} {
		set alertstatus true
		append alertreasons "High Temp - "
		if {$graphicson} {
			writehtml "<tr><td width=\"80\"><center><img src=\"[imageref $emergencytempfilename]\" border=\"0\" alt=\"Temp Info\"></center></td>"
			writehtml "<td width=\"[expr $programbarwidthpx + 170]\""
		} else {
			writehtml "<tr><td width=\"[expr $programbarwidthpx + 250]\""
		}
		if { $digit_tempC > $hightempcentigrade} {
			writehtml " class=Warning>WARNING: The TiVo temperature is at high levels - currently $cur_temp / $cur_tempF ! "
			dmdebug 1 "WARNING-Temperature is HIGH - currently $cur_temp / $cur_tempF."
		} elseif { $digit_tempC > $toohightempcentigrade} {
			writehtml " class=Warning>WARNING: The TiVo temperature is at very high level - currently $cur_temp / $cur_tempF ! Please investigate immediately immediately!"
			dmdebug 1 "WARNING-temperature is TOO HIGH - currently $cur_temp / $cur_tempF - PLEASE INVESTIGATE!"
		} elseif { $digit_tempC > $emergencytempcentigrade} {
			writehtml " class=Warning>WARNING: The TiVo temperature is at dangerous levels - currently $cur_temp / $cur_tempF ! Turn off your TiVo immediately!"
			dmdebug 1 "DANGER termperature is DANGEROUSLY HIGH - currently $cur_temp / $cur_tempF - TURN OFF MACHINE NOW!"
		}
		if {$graphicson} {} else { writehtml "<br><br>" }
		writehtml "</td></tr></table>"	
	}
	
	catch { unset cur_tempF digit_tempC  cur_temp }
	dmdebug 1 "CHECKING FOR EMERGENCY TEMP READING - COMPLETED"
}
	
# END 100.
# #######################################################################################################


# #######################################################################################################
# 110. Checking phone & daily call status

if 1 {	
	dmdebug 1 "CHECKING PHONE & DAILY CALL STATUS - START"
	
	writehtml "<table width=\"[expr $programbarwidthpx + 250]\" border=\"0\" class=indented>"
	set phone [phone_status]
	if {$bulletinverbose || $phone != "Ok"} {
		if {$phone != "Ok"} {
			set ourimagefilename $phonebrokenfilename
			set alertstatus true
			append alertreasons "Phone Problems - "
		} else {
			set ourimagefilename $phonefilename
		}
		if {$graphicson} {
			if {$dmailLinksOn} {
				writehtml "<tr><td width=\"80\"><center><a href=\"$dmailTivoURL/Phone/\" target=\"_blank\" ><img src=\"[imageref $ourimagefilename]\" border=\"0\" title=\"Click&nbsp;here&nbsp;to&nbsp;view&nbsp;further&nbsp;phone&nbsp;information.\" alt=\"Phone Info\"></a></center></td>"	
			} else {
				writehtml "<tr><td width=\"80\"><center><img src=\"[imageref $ourimagefilename]\" border=\"0\" alt=\"Phone Info\"></center></td>"
			}
			writehtml "<td width=\"[expr $programbarwidthpx + 170]\""
		} else {
			writehtml "<tr><td width=\"[expr $programbarwidthpx + 250]\""
		}
		if {$phone != "Ok"} {
			writehtml " class=Warning>WARNING: Last daily call failed with error of $phone."
		} else {
			writehtml " class=Normal>Last daily call was performed without error."
		}
		if {$graphicson} {} else { writehtml "<br><br>" }
		writehtml "</td></tr></table>"	
	}
	
	catch { unset ourimagefilename phone }
	dmdebug 1 "CHECKING PHONE & DAILY CALL STATUS - COMPLETED"
}

# END 110.
# #######################################################################################################


# #######################################################################################################
# 120. SMART Disk Check

if 1 {
	dmdebug 1 "SMART DISK CHECK - START"
	
	writehtml "<table width=\"[expr $programbarwidthpx + 250]\" border=\"0\" class=indented>"
	set passeda 0
	set passedb 0
	if { $smartCmd != "" } {
		dmdebug 2 "Checking S.M.A.R.T. status"
		# /var/hack/bin/smartctl -a /dev/hda
		if { [ catch { set smart1msg [ exec $smartCmd -a /dev/hda ] } ] } {
			set passeda 1
			dmdebug 1 "SMART command did not successfully run on disk A - check path & setup."
			dmerrorinfo 1 $errorInfo
		} else {
			dmdebug 2 "Drive A SMART check :"
			#
			# smartctl version 1.2 returns "Check S.M.A.R.T. Passed"
			#          version 5.1-9 returns "SMART overall-health self-assessment test result: PASSED"
			#
			if { ( [string first "test result: PASSED" $smart1msg] > -1 ) || \
					 ( [string first "Check S.M.A.R.T. Passed" $smart1msg] > -1 ) } { 
				set passeda 1
				dmdebug 2 "disk passed SMART check."
				set lvl 3
			} else {
				dmdebug 2 "disk failed SMART check - details shown here :"
				set lvl 2
			}
			dmdebug $lvl "********************************************************************************" false
			dmdebug $lvl "$smart1msg" false
			dmdebug $lvl "********************************************************************************" false
			unset lvl
		}
		if {$diskBpresent} {
			dmdebug 2 "Disk B is present - run SMART check."
			if { [ catch { set smart2msg [ exec $smartCmd -a /dev/hdb ] } ] } {
				set passedb 1
				dmdebug 1 "SMART command did not successfully run on disk B - check path & setup - or you do not have a second hard drive."
				dmerrorinfo 0 $errorInfo
				if {$::debuglvl>0} { puts "----------------------------------------------------------------"}
			} else {
				dmdebug 2 "Drive B SMART check :"
				if { ( [string first "test result: PASSED" $smart2msg] > -1 ) || \
					 ( [string first "Check S.M.A.R.T. Passed" $smart2msg] > -1 ) } {
					set passedb 1
					dmdebug 2 "disk passed SMART check."
					set lvl 3
				} else {
					dmdebug 2 "disk failed SMART check - details shown here :"
					set lvl 2
				}
				dmdebug $lvl "********************************************************************************" false
				dmdebug $lvl "$smart2msg" false
				dmdebug $lvl "********************************************************************************" false
				unset lvl
			}
		} else {
			dmdebug 2 "Disk B is not present - SMART check not run on disk B - remember disk B presence is set in the cfg file."
			set passedb 1
		}
	}
	
	dmdebug 1 "SMART DISK CHECK - COMPLETED"
}

# END 120
# #######################################################################################################

	
# #######################################################################################################
# 130. Space Usage Display

if 1 {	
	dmdebug 1 "CALCULATING DISK USAGE & DISPLAY - START"
	
	# dm_space_details returns an array of disk info, e.g.
	array set diskinfo {}
	dm_space_details diskinfo
	if {$::debuglvl >= 4} { printarray ::diskinfo }

	dmdebug 1 "creating mouseover space display"
	set spacedetail "Space usage breakdown: \n"
	for {set type 1} {$type <= 17} {incr type} {
			foreach {datatype value} [array get diskinfo type$type] {
				if {$datatype != ""} {
					set ourtemptype [get_type $type]
					dmdebug 3 "($type) $ourtemptype = $value MB"
					append spacedetail "- $ourtemptype :  $value  MB \n"
				}
			}
			foreach {datatype value} [array get diskinfo type-$type] {
				if {$datatype != ""} {
					set ourtemptype [get_type -$type]
					dmdebug 3 "(-$type) $ourtemptype = $value MB"
					append spacedetail "- $ourtemptype :  $value  MB \n"
				}
			}
		}
		# live cache (i.e. type = 100)
		dmdebug 3 "(O) Live Cache = $diskinfo(type100) MB"
		append spacedetail "- Live Cache : $diskinfo(type100) MB \n"	
		# total used
		dmdebug 3 "(-) TOTAL = $diskinfo(totalused) MB"
		append spacedetail "- Total : $diskinfo(totalused) MB \n"
		# free space percentage
		dmdebug 3 "(-) % = [expr 100 - $diskinfo(freepercent)] %"
		append spacedetail "- % Used : [expr 100 - $diskinfo(freepercent)] % \n"
		# convert to JS alert
		set spacepopdetail [nl2crnl $spacedetail]
		

		dmdebug 1 "calculating working values"	
		if {$::version >= 3} {
			set namearray $presentationbehtypes
		} else {
			set namearray $selectiontypes
		}
		set spaceusedbysuggestionsmeg 0 							;# suggestions 
		set spaceusedbyexpiredsuggestionsmeg 0				;# expired suggestions 
		set spaceusedbyexpiredmeg 0										;# expired - not including suggestions
		set corespaceusedmeg 0												;# excluding suggestions & expired
		set suggindex [expr [lsearch $namearray {Suggestion}]+1]
    catch { set spaceusedbysuggestionsmeg "$diskinfo(type$suggindex)" }
		catch { set spaceusedbyexpiredsuggestionsmeg "$diskinfo(type-$suggindex)" }
		for {set type 1} {$type <= 17} {incr type} {
			if {$type != $suggindex} {
				catch {	set spaceusedbyexpiredmeg [expr $spaceusedbyexpiredmeg + "$diskinfo(type-$type)"] }
			}
		}
		set corespaceusedmeg [expr $diskinfo(totalused)  - $spaceusedbysuggestionsmeg - $spaceusedbyexpiredsuggestionsmeg - $spaceusedbyexpiredmeg]
		dmdebug 3 "(O) Un-Expired suggestions = $spaceusedbysuggestionsmeg MB"
		dmdebug 3 "(O) Expired suggestions = $spaceusedbyexpiredsuggestionsmeg MB"
		dmdebug 3 "(O) Expired items = $spaceusedbyexpiredmeg MB"
		dmdebug 3 "(O) Core Used = $corespaceusedmeg MB"
		
		if {$::expiredsuggestionstreatment == "S"} {
			dmdebug 3 "-expired suggestions are classified suggestions"
    	set spaceusedbysuggestionsmeg [expr $spaceusedbysuggestionsmeg + $spaceusedbyexpiredsuggestionsmeg]		
		} elseif {$::expiredsuggestionstreatment == "E"} {
			dmdebug 3 "-expired suggestions are classified expired items"
			set spaceusedbyexpiredmeg [expr $spaceusedbyexpiredmeg + $spaceusedbyexpiredsuggestionsmeg] 		
		} else {					; same as "S" !
			dmdebug 3 "-expired suggestions are classified suggestions"
			set spaceusedbysuggestionsmeg [expr $spaceusedbysuggestionsmeg + $spaceusedbyexpiredsuggestionsmeg]
		}  		
    dmdebug 3 "-space used by classified suggestions = $spaceusedbysuggestionsmeg meg"
		dmdebug 3 "-space used by classified expired items = $spaceusedbyexpiredmeg meg"
		
		set interpretedspaceusedmeg 0
		if {$::countsuggestionsasfreespace && $::countexpiredasfreespace} {
			dmdebug 3 "-suggestions & expired are free space"
			set interpretedspaceusedmeg  $::corespaceusedmeg
			set interpretedotherspacemeg [expr $spaceusedbysuggestionsmeg + $spaceusedbyexpiredmeg]
		} else {
			if {$::countsuggestionsasfreespace} {
				dmdebug 3 "-suggestions are free space - expired items are used space"
				set interpretedspaceusedmeg  [expr $::corespaceusedmeg + $spaceusedbyexpiredmeg]
				set interpretedotherspacemeg $spaceusedbysuggestionsmeg  			
			} elseif {$::countexpiredasfreespace} {
				dmdebug 3 "-expired items are free space - suggestions are used space"
				set interpretedspaceusedmeg  [expr $::corespaceusedmeg + $spaceusedbysuggestionsmeg]
				set interpretedotherspacemeg $spaceusedbyexpiredmeg
			} else {
				dmdebug 3 "-suggestions & expired items are used space"
				set interpretedspaceusedmeg  [expr $::corespaceusedmeg + $spaceusedbysuggestionsmeg + $spaceusedbyexpiredmeg ]
				set interpretedotherspacemeg 0	
			}
		}
		dmdebug 3 "-interpreted space used = $interpretedspaceusedmeg meg"
		dmdebug 3 "-interpreted other space = $interpretedotherspacemeg"
    		
		set interpretedfreespacemeg [expr $diskinfo(totalspace) - $interpretedspaceusedmeg]
		set interpretedpercentused [expr ((($interpretedspaceusedmeg*1.0000) / ($diskinfo(totalspace)*1.0000))*100.00)]
		dmdebug 3 "-interpreted free space = $interpretedfreespacemeg meg"
		dmdebug 3 "-interpreted percent used = $interpretedpercentused"

		set actualdrivespacemeg  [expr round($diskinfo(totalspace))]
		set actualspaceusedmeg [expr round($interpretedspaceusedmeg)]
		set actualspacefreemeg [expr round($interpretedfreespacemeg)]
		set otherspacemeg [expr round($interpretedotherspacemeg)]
		set actualpercentused [expr round($interpretedpercentused * 100) / 100.00]
		set actualpercentfree [expr 100.00 - $actualpercentused]
		set otherpercent [expr round((((($interpretedotherspacemeg*1.0000) / ($diskinfo(totalspace)*1.0000))*100.00)) * 100) / 100.00]
		set actualdrivespacegig  [expr round(($actualdrivespacemeg/1024) * 10) / 10.0]	
		set actualspaceusedgig [expr round(($actualspaceusedmeg/1024) * 10) / 10.0]	
		set actualspacefreegig  [expr round(($actualspacefreemeg/1024) * 10) / 10.0]		
		set otherspacegig  [expr round(($otherspacemeg/1024) * 10) / 10.0]
		dmdebug 3 "-core space used = $corespaceusedmeg meg"
		dmdebug 3 "-actual drive space = $actualdrivespacemeg meg"
		dmdebug 3 "-actual space used = $actualspaceusedmeg meg"
		dmdebug 3 "-actual space free = $actualspacefreemeg meg"
		dmdebug 3 "-other space = $otherspacemeg meg"
		dmdebug 3 "-actual percent used = $actualpercentused %"
		dmdebug 3 "-actual percent free = $actualpercentfree %"
		dmdebug 3 "-other percent = $otherpercent %"
} 


if 1 {	
	## SMART status
	set rndpercentused [ expr round ( $actualpercentused ) ]
	## check to see if we need to show bulletin
	if {$bulletinverbose || $passeda == 0 || $passedb == 0 || $rndpercentused > $highthreshold } {
		## always show graphic
		if {$rndpercentused > $highthreshold} {
			set alertstatus true
			append alertreasons "Disk Issues - "
			if { $passeda == 0 || $passedb == 0 } {
				set ourimagefilename $diskfullbrokenfilename
			} else {
				set ourimagefilename $diskfullfilename
			}
		} else {
			if { $passeda == 0 || $passedb == 0 } {
				set ourimagefilename $diskbrokenfilename
				set alertstatus true
				append alertreasons "Disk Issues - "
			} else {
				set ourimagefilename $diskfilename
			}
		}
		if {$graphicson} {
			writehtml "<tr><td width=\"80\"><center><img src=\"[imageref $ourimagefilename]\" border=\"0\" alt=\"Disk Info\"></center></td><td width=\"[expr $programbarwidthpx + 170]\""
		} else {
			writehtml "<tr><td width=\"[expr $programbarwidthpx + 250]\""
		}
		
		## sort out message
		if { $passeda ==  0 || $passedb == 0 || $rndpercentused > $highthreshold } {
			writehtml " class=Warning>WARNING: " 1
		} else {
			writehtml " class=Normal>" 1
		}
		if { $smartCmd != "" } {
			if { $passeda == 0 } {
				# if SMART errors are detected, then print them out in the email
				writehtml "SMART errors detected on drive A: <pre>$smart1msg</pre>" 1
			} 
			if {$diskBpresent} {
				if { $passedb == 0 } {
					# If SMART errors are detected, then print them out in the email
					writehtml "SMART errors detected on drive B: <pre>$smart2msg</pre>" 1
				}
				if { $passeda && $passedb } {
					writehtml "No SMART errors detected. " 1
				}
			} else {
				if {$passeda} {
					writehtml "No SMART errors detected. " 1
				}
			}
		} 
		catch { unset passeda passedb smart1msg smart2msg }
		## end SMART status
		
		## disk usage
		if { $smartCmd != "" } {
			if { $rndpercentused > $highthreshold } {
				writehtml "Usage is high ($rndpercentused%). " 1
			} else {
				writehtml "Usage is at $rndpercentused%. " 1
			}
		} else {
			if { $rndpercentused > $highthreshold } {
				writehtml "Drive usage is high ($rndpercentused%). " 1
			} else {
				writehtml "Drive usage is at $rndpercentused%. " 1
			}
		}
		## end disk usage
		
		## remaining time
		set ourunitstext [recqualitytext $defrecquality lower]
		regexp {^([0-9]+):([0-9]+):([0-9]+)$} $diskinfo(freetime) junk ourhours ourminutes ourseconds
		if {$diskinfo(freetime) <= 0} {
			set ourremainingtimetext  "TiVo has no recording time remaining." 
		} else {
			set ourremainingtimetext  "TiVo has an estimated [expr ($ourhours > 0)?"$ourhours hours":""] ${ourminutes} mins<sup>*</sup> of $ourunitstext quality time remaining."
			set remainingtimenotice 1
		}
		dmdebug 2 $ourremainingtimetext 
		writehtml "$ourremainingtimetext " 1
		unset  ourunitstext
		## end remaining time	

		## space bar
		if {$spacebarrequired} {
			writehtml " "
			dmdebug 2 "Building usage diagram"
			writehtml "<br /><table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" align=centre width=1%><tr>"
			set lowsize 0; set midsize 0; set highsize 0; set extrasize 0; set remaindersize 0
			set lowsize [min $rndpercentused $midthreshold]
			set highsize [max 0 [expr $rndpercentused - $highthreshold]]
			set midsize [max 0 [expr $rndpercentused - $midthreshold - $highsize]]
			dmdebug 3 "lowsize >>$lowsize<< | midsize >>$midsize<< | highsize >>$highsize<<"
			
			if {$spacebarsuggexpshow && ($countsuggestionsasfreespace || $countexpiredasfreespace)} {
				dmdebug 3 "suggestions / expired treated as free space (A)."
				# if we want to show suggestions in space bar and either suggestions or expired items are being treated as used space
				set extrasize [expr round($otherpercent)]
				if {$extrasize == 0 && $otherpercent != 0 } { set extrasize 1 }
				set remaindersize [ expr 100 - ($actualpercentused + $extrasize) ]
			} else {
				set remaindersize [ expr 100 - $actualpercentused ]
			}
			set multipletime $graphiclengthmultiple
			set lowsize [expr $lowsize * $multipletime]
			set midsize [expr $midsize * $multipletime]
			set highsize [expr $highsize * $multipletime]	
			set extrasize [expr $extrasize * $multipletime]
			set remaindersize [expr $remaindersize * $multipletime]
			
			if {$spacebarsuggexpshow} {
				dmdebug 3 "suggestions / expired treated as free space (D)."
				# if we want to show a space bar
				if {$countsuggestionsasfreespace && $countexpiredasfreespace} {
					dmdebug 3 "-both."
					set drivetext "Drive usage is at ${actualpercentused}% not including suggestions and expired items.\nTotal available drive space is ${actualdrivespacegig} Gb.\nActual space used is ${actualspaceusedgig} Gb.\nExpired items and suggestions use an additional ${otherspacegig} Gb.\n${ourremainingtimetext}"					
				} else {
					if {$countsuggestionsasfreespace} {
						dmdebug 3 "-just suggestions are free space."
						# if expired items are being treat as used space
						set drivetext "Drive usage is at ${actualpercentused}% not including suggestions.\nTotal available drive space is ${actualdrivespacegig} Gb.\nActual space used is ${actualspaceusedgig} Gb.\nSuggestions use an additional ${otherspacegig} Gb.\n${ourremainingtimetext}"
					} elseif {$countexpiredasfreespace} {
						dmdebug 3 "-just expired are free space."
						# if suggestions are being treat as used space 
						set drivetext "Drive usage is at ${actualpercentused}% not including expired items.\nTotal available drive space is ${actualdrivespacegig} Gb.\nActual space used is ${actualspaceusedgig} Gb.\nExpired items use an additional ${otherspacegig} Gb.\n${ourremainingtimetext}"
					} else {
						dmdebug 3 "-weird!"
						set drivetext "Drive usage is at ${actualpercentused}.\nTotal available drive space is ${actualdrivespacegig} Gb.\nActual space used is ${actualspaceusedgig} Gb.\n${ourremainingtimetext}"
					}
				}
			} else {
				dmdebug 3 "-not concerned with suggestions and expired display"
				set drivetext "Drive usage is at ${actualpercentused}.\nTotal available drive space is ${actualdrivespacegig} Gb.\nActual space used is ${actualspaceusedgig} Gb.\n${ourremainingtimetext}"
			}

			# convert to JS alert
			set drivepoptext [nl2crnl $drivetext]
					
			# write the space bar
			writehtml "<td nowrap><a style=\"text-decoration:none\" href=\"#\">" 1
			spacegraphic $spacegraphicstart 1 "" "|" "begin"
			spacegraphic $spacegraphiclow $lowsize "#339966" "Low"			
			spacegraphic $spacegraphicmid $midsize "yellow" "Medium"		
			spacegraphic $spacegraphichigh $highsize "maroon" "High"			
			spacegraphic $spacegraphicextra $extrasize "silver" "Other"			
			spacegraphic $spacegraphicempty $remaindersize "white" "Space"				
			spacegraphic $spacegraphicend 1 "" "|" "end"
			writehtml "</a></td><td>&nbsp;&nbsp;</td>"
			
			if {$graphicson} {
				if {$popuptextrequired} {
					writehtml "<td><a href=\"javascript:void(0);\" onclick=\"alert('${spacepopdetail}')\"><img src=\"[imageref $spacebarinfographic]\" border=\"0\" height=\"12\" vspace=\"0\" hspace=\"0\" title=\"${spacedetail}\" alt=\"i\"></a></td>"
				} else {
					writehtml "<td><img src=\"[imageref $spacebarinfographic]\" border=\"0\" height=\"12\" vspace=\"0\" hspace=\"0\" title=\"${spacedetail}\" alt=\"i\"></td>"
				}
			}
			
			writehtml "</tr></table>"		
		}
		## end space bar
		
	
		if {$graphicson} {} else { writehtml "<br><br>" }
		writehtml "</td></tr></table>"	
	}
	
	catch { unset drivetext drivepoptext lowsize midsize highsize extrasize ourhours ourminutes ourseconds ourimagefilename ourremainingtimetext multipletime}
	dmdebug 1 "CALCULATING DISK USAGE & DISPLAY - COMPLETED"
}

# END 130. 
# #######################################################################################################

			
# #######################################################################################################
# 140.ENDPAD CHECKING SECTION

if { $endpadinstalled && $endpadstatus } {
	dmdebug 1 "ENDPAD CHECKING - START"
	
	# First check the pid file and grab the process number from it
	set endpadrunning ""
	set i 0
	while {$i < 5 && $endpadrunning != "ok" && $endpadrunning != "??"} {
		if { $i > 0 } {
			dmdebug 1 "checking status of the endpad daemon again - attempt [expr $i + 1]."
			dmdebug 1 "waiting for 15 seconds."
			after 15000
		}
		dmdebug 3 "getting pid number from endpad pid file."
		if {[catch {
			set pidfile [open "$pidfilelocation" "r"]
			while { ! [eof $pidfile] } {
				set pidline [gets $pidfile]
				if {$pidline  != ""} {
					set pidnumber $pidline
				} 
			}
		}]} {
			set pidnumber ""
			dmdebug 2 "error gathering pid number : endpad pid file may not exist."
		} else {
			dmdebug 3 "endpad pid number gathered successfully."
		}
		dmdebug 3 "pid number : >>$pidnumber<<"
		# If we have a pid number then lets do a ps to see if the process is running	
		if {$pidnumber != ""} {
			dmdebug 3 "performing ps command with pid"
			set shellcmd "$pslocation"
			dmdebug 3 "the command being executed is >>exec $shellcmd $pidnumber<<."
			if {[catch { exec $shellcmd $pidnumber } shellmsg]} {
				dmdebug 2 "ps command was unsuccessful."
			} else {
				dmdebug 3 "ps command successful."
				dmdebug 3 "endpadexecfilename: >>$endpadexecfilename<<."
				dmdebug 3 "shellmsg: >>$shellmsg<<."
				if { [string first $endpadexecfilename $shellmsg] > -1 } {
					dmdebug 3 "successfully found valid endpad filename in ps output"
					# check the endpad log stats and ensure that within a 5 minute window for valid running criteria
					dmdebug 3 "checking endpad log to ensure it has been modified within the last 5 minutes."
					if {! [catch {file stat $endpadlogloc endpadlogstats} ] } {
						dmdebug 3 "modification time of endpad log file : >>$endpadlogstats(mtime)<<."
						if {[expr [clock seconds] - $endpadlogstats(mtime)] < 301 } {
							set endpadrunning "ok"
							dmdebug 2 "endpad daemon running ok."
						} else {
							dmdebug 2 "endpad daemon not running ok - ouside 5 minute update window"
						}
					} else {
						dmdebug 2 "problem - cannot collect stats about endpad log file."
					}
				} else {
					dmdebug 2 "error - the valid filename is not in the ps command output"
					dmdebug 2 "output of command : >>$shellmsg<<"
					# if the process is sleeping then we won't get the filename in the ps output (we get generic (tivosh) instead
					if {[regexp {\(tivosh\)} $shellmsg] } {
						dmdebug 2 "endpad may be sleeping"
						set endpadrunning "??"
					}
				}	
			}
		}
		incr i
	}
	if {$bulletinverbose || $endpadrunning != "ok"} {
		writehtml "<table width=\"[expr $programbarwidthpx + 250]\" border=\"0\" class=indented>"
		if { $endpadrunning == "ok" } {
			set ourimagefilename $endpadonfilename
			set pidtext "Process Id: $pidnumber."
		} else {
			set ourimagefilename $endpadofffilename
			set pidtext ""
		}		
		if {$graphicson} {
			if {$dmailLinksOn} {
				writehtml "<tr><td width=\"80\"><center><a href=\"$dmailTivoURL/controlpadding/\" target=\"_blank\"><img src=\"[imageref $ourimagefilename]\" border=\"0\" title=\"$pidtext Click here to start and stop soft padding.\" alt=\"Padding Info\"></a></center></td><td width=\"[expr $programbarwidthpx + 170]\""
			} else {
				writehtml "<tr><td width=\"80\"><center><img src=\"[imageref $ourimagefilename]\" border=\"0\" title=\"$pidtext\" alt=\"Padding Info\"></center></td><td width=\"[expr $programbarwidthpx + 170]\""
			}
		} else {
			writehtml "<tr><td width=\"[expr $programbarwidthpx + 250]\"" 1
		}		
		if { $endpadrunning == "ok" } {
			writehtml " class=Normal>Soft padding is active.</td></tr>"
		} elseif { $endpadrunning == "??" } {
			writehtml " class=Warning>Soft padding status unknown.</td></tr>"
		} else {
			writehtml " class=Warning>WARNING: Soft padding is not currently active!</td></tr>"
			set alertstatus true
			append alertreasons "Padding Stopped - "
		}
		if {$graphicson} {} else { writehtml "<br><br>" }
		writehtml "</td></tr></table>"	
	}
	
	catch { unset endpadlogstats endpadrunning shellcmd pidnumber pidline pidfile pidtext ourimagefilename}
	dmdebug 1 "ENDPAD CHECKING - COMPLETED"
}

# END 140.
# #######################################################################################################


# #######################################################################################################
# 150. RED DOT CHECKING

if { $reddotinstalled && $reddotstatus } {
	dmdebug 1 "RED DOT CHECKING - START"

	# check the process
	set reddotrunning ""
	set shellcmd "$pslocation"
	if {[catch { exec $shellcmd ax } shellmsg]} {
		# null
	} else {
		if { [string first $noreddotexecfilename  $shellmsg] > -1 } {
			set reddotrunning "ok"
		}
	}
	if {$bulletinverbose || $reddotrunning!= "ok"} {
		writehtml "<table width=\"[expr $programbarwidthpx + 250]\" border=\"0\" class=indented>"
		if { $reddotrunning == "ok" } {
			set ourimagefilename $reddotonfilename
			set titletext ""
		} else {
			set ourimagefilename $reddotofffilename
			set titletext "You will need to restart noreddot at the command line."
		}					
		if {$graphicson} {
			if {$dmailLinksOn} {
				writehtml "<tr><td width=\"80\"><center><img src=\"[imageref $ourimagefilename]\" border=\"0\" title=\"$titletext\" alt=\"Red Dot Info\"></center></td><td width=\"[expr $programbarwidthpx + 170]\""
			} else {
				writehtml "<tr><td width=\"80\"><center><img src=\"[imageref $reddotofffilename]\" border=\"0\" title=\"$titletext\" alt=\"Red Dot Info\"></center></td><td width=\"[expr $programbarwidthpx + 170]\""
			}
		} else {
			writehtml "<tr><td width=\"[expr $programbarwidthpx + 250]\"" 1
		}
		if { $reddotrunning == "ok" } {
			writehtml " class=Normal>Red dot blast is active.</td></tr>"			
		} else {				
			writehtml " class=Warning>WARNING: Red dot blast is not currently active!</td></tr>"
			set alertstatus true
			append alertreasons "Red Dot Remover Stopped - "
		}
		if {$graphicson} {} else { writehtml "<br><br>" }
		writehtml "</td></tr></table>"
	}

	catch { unset shellmsg shellcmd titletext ourimagefilename}
	dmdebug 1 "RED DOT CHECKING - COMPLETED"
}

# END 150
# #######################################################################################################


# #######################################################################################################
# 160. NEW MAIL SECTION

if { $mailcheckingon } {
	dmdebug 1 "NEW MAIL SECTION - START"
	
	dmdebug 1 "checking for new messages in tivo central"
	set gotmail [dmcheckmail]
	if { $gotmail } {
		writehtml "<table width=\"[expr $programbarwidthpx + 250]\" border=\"0\" class=indented>"
		if {$graphicson} {
			if {$dmailLinksOn} {
				writehtml "<tr><td width=\"80\"><center><a href=\"$dmailTivoURL/$linkmail\" target=\"_blank\"><img src=\"[imageref $mailfilename]\" border=\"0\" alt=\"Mail Info\" title=\"Click here to view new mails.\" ></a></center></td><td width=\"[expr $programbarwidthpx + 170]\""
			} else {
				writehtml "<tr><td width=\"80\"><center><img src=\"[imageref $mailfilename]\" border=\"0\" alt=\"Mail Info\"></center></td><td width=\"[expr $programbarwidthpx + 170]\""
			}
		} else {
			writehtml "<tr><td width=\"[expr $programbarwidthpx + 250]\"" 1
		}
		if {$dmailLinksOn} {
			writehtml " class=Warning>ALERT: You have new mail (<a href=\"$dmailTivoURL/$linkmail\" target=\"_blank\" title=\"Click here to view new mails.\">view</a>)."
		} else {
			writehtml " class=Warning>ALERT: You have new mail."
		}
		set alertstatus true
		append alertreasons "New Mail - "
		if {$graphicson} {} else { writehtml "<br><br>" }
			writehtml "</td></tr></table>"
	}

	catch { unset gotmail }
	dmdebug 1 "NEW MAIL SECTION - COMPLETED"
}
	
# END 160
# #######################################################################################################


# #######################################################################################################
# 170. MAIN EMAIL BODY SECTION

if 1 {
	dmdebug 1 "MAIN EMAIL BODY - START"
	
	writehtml "</table>"
	if {$graphicson} {} else { writehtml "<br>" }
	dmdebug 3 "CALLING SECTION #1"
	if { [catch { outputSection 1 } fid] } {
		dmdebug 3 "ERROR in section #1 >>$fid<<"
		dmerrorinfo 0 $errorInfo
	}
	dmdebug 3 "CALLING SECTION #2"
	if { [catch { outputSection 2 } fid] } {
		dmdebug 3 "ERROR in section #2 >>$fid<<"
		dmerrorinfo 0 $errorInfo
	}
	dmdebug 3 "CALLING SECTION #3"
	if { [catch { outputSection 3 } fid] } {
		dmdebug 3 "ERROR in section #3 >>$fid<<"
		dmerrorinfo 0 $errorInfo
	}
	dmdebug 3 "CALLING SECTION #4"
	if { [catch { outputSection 4 } fid] } {
		dmdebug 3 "ERROR in section #4 >>$fid<<"
		dmerrorinfo 0 $errorInfo
	}
	dmdebug 3 "CALLING SECTION #5"
	if { [catch { outputSection 5 } fid] } {
		dmdebug 3 "ERROR in section #5 >>$fid<<"
		dmerrorinfo 0 $errorInfo
	}
	dmdebug 3 "CALLING SECTION #6"
	if { [catch { outputSection 6 } fid] } {
		dmdebug 3 "ERROR in section #6 >>$fid<<"
		dmerrorinfo 0 $errorInfo
	}
	
	dmdebug 1 "MAIN EMAIL BODY - COMPLETED"
}

# END 170.
# #######################################################################################################
	

# #######################################################################################################
# 180.  EMAIL FOOTER SECTION
	
if 1 {
	dmdebug 1 "MAIN FOOTER - START"
	
	if {$dmailLinksOn} {
		writehtml "<br><br>"
		if {[info exists remainingtimenotice] && $remainingtimenotice} {
			writehtml "<p><sup>*</sup>Note: This uses an average bit rate in its calculations - this is only an approximation of future recordings and may be wildly inaccurate e.g. if you have recently switched recording quality, or have just switched to HD, etc. DO NOT rely on this value! </p>"
		}
		writehtml "<p>Visit $tivoName TiVo's <a href=\"$dmailTivoURL/\" title=\"Click here to view your TiVo's home page.\">Home Page</a></p>"
		set twotimeshours [expr $confighoursahead * 2]
		set threetimeshours [expr $confighoursahead * 3]
		set fourtimeshours [expr $confighoursahead * 4]
		set fivetimeshours [expr $confighoursahead * 5]
		set oncehoursloc "$dmailTivoURL/custommail?optmsg=This+daily+email+was+requested+manually+from+a+previously+sent+daily+email.&recipient=$recipient&hoursahead=$confighoursahead&submit=Send+Email"
		set twotimeshoursloc "$dmailTivoURL/custommail?optmsg=This+daily+email+was+requested+manually+from+a+previously+sent+daily+email.&recipient=$recipient&hoursahead=$twotimeshours&submit=Send+Email"
		set threetimeshoursloc "$dmailTivoURL/custommail?optmsg=This+daily+email+was+requested+manually+from+a+previously+sent+daily+email.&recipient=$recipient&hoursahead=$threetimeshours&submit=Send+Email"
		set fourtimeshoursloc "$dmailTivoURL/custommail?optmsg=This+daily+email+was+requested+manually+from+a+previously+sent+daily+email.&recipient=$recipient&hoursahead=$fourtimeshours&submit=Send+Email"
		set fivetimeshoursloc "$dmailTivoURL/custommail?optmsg=This+daily+email+was+requested+manually+from+a+previously+sent+daily+email.&recipient=$recipient&hoursahead=$fivetimeshours&submit=Send+Email"
		writehtml "<p>Resend updated email for <a href=\"$oncehoursloc\" target=\"_blank\">$confighoursahead</a>, <a href=\"$twotimeshoursloc\" target=\"_blank\">$twotimeshours</a>, "
		writehtml "<a href=\"$threetimeshoursloc\" target=\"_blank\">$threetimeshours</a>, <a href=\"$fourtimeshoursloc\" target=\"_blank\">$fourtimeshours</a> or <a href=\"$fivetimeshoursloc\" target=\"_blank\">$fivetimeshours</a> hours ahead.</p>"
	}
	
	writehtml "<br><i>jazzed dailymail v$software_version ($software_release_date) </i><br>"
	closehtml
	
	catch { unset twotimeshours threetimeshours fourtimeshours fivetimeshours oncehoursloc twotimeshoursloc threetimeshoursloc fourtimeshoursloc fivetimeshoursloc }
	dmdebug 1 "MAIN FOOTER - COMPLETED"
}
	
# 180 END
# #######################################################################################################


dbclose $db


# #######################################################################################################
# 190. SEND EMAILS & CREATE TEXT FILE

if 1 {
	dmdebug 1 "EMAIL & HTML FILE MANAGEMENT - START"
	if {$tivoName==""} {
		set tempTiVoName ""
	} else {
		set tempTiVoName "$tivoName "
	}
	if {$datedsubjectreq} {
		if {$alertonlyemail} {
			set ourSubject "${tempTiVoName}TiVo Alert Email For [clock format [expr [clock seconds] + $tzoffset] -format {%A %B %d, %Y} -gmt true]"
		} else {
			set ourSubject "${tempTiVoName}TiVo Status Email For [clock format [expr [clock seconds] + $tzoffset] -format {%A %B %d, %Y} -gmt true]"
		}
	} else {
		if {$alertonlyemail} {
			set ourSubject "${tempTiVoName}TiVo Alert"
		} else {
			set ourSubject "${tempTiVoName}TiVo Status"
		}
	}
	if {$alertonlyemail} {
		dmdebug 1 "EMAIL WILL BE SENT ON ALERT ONLY : STATUS >>$alertstatus<<"
		dmdebug 2 "ALERT REASONS : >>$alertreasons<<"
	}
	
	set doemail false
	if {$alertstatus && $alertonlyemail} {
		## if it is defined that we only send an email on alert and alter exists then go for it
		set doemail true
		dmdebug 4 "email checkpoint #1"
	}
	if {$alertonlyemail} {
		## nothing
	} else {
		## if alert only email is not switched on then send email anyway
		set doemail true
		dmdebug 4 "email checkpoint #2"
	}
	
	if {$doemail} {
		if {$emailsendrequired} {
			dmdebug 1 "sending e-mail"
			
			if { [catch { sendmail $mailserver $domain $recipient "$ourSubject" $htmlfilename } fid] } {
				dmdebug 1 "FAILURE WHEN CALLING SENDMAIL FOR FIRST RECIPIENT >>$fid<<"
				dmerrorinfo 0 $errorInfo	
			} else {
				dmdebug 1 "e-mail sent"
			}
			if { $recipient2 != "" } {
				dmdebug 2 "sending e-mail to recipient 2"
				if { [catch { sendmail $mailserver $domain $recipient2 "$ourSubject" $htmlfilename } fid] } {
					dmdebug 1 "FAILURE WHEN CALLING SENDMAIL FOR SECOND RECIPIENT >>$fid<<"
					dmerrorinfo 0 $errorInfo
				} else {
					dmdebug 1 "e-mail sent to recipient 2"
				}	
			}
			if { $recipient3 != "" } {
				dmdebug 1 "sending e-mail to recipient 3"
				if { [catch { sendmail $mailserver $domain $recipient3 "$ourSubject" $htmlfilename } fid] } {
					dmdebug 1 "FAILURE WHEN CALLING SENDMAIL FOR THIRD RECIPIENT >>$fid<<"
					dmerrorinfo 0 $errorInfo
				} else {
					dmdebug 1 "e-mail sent to recipient 3"	
				}
			}
		} else {
			dmdebug 1 "e-mail not required"
		}
	} else {
		dmdebug 1 "EMAIL NOT SENT BECAUSE THERE IS NO ALERT DISCOVERED!"
	}
	catch { unset doemail }
	if {$htmlfilerequired} {
		## nothing
	} else {
		dmdebug 1 "HTML file generation not required"
		file delete -force $htmlfilename
	}
	
	catch { unset tempTiVoName }
	
	dmdebug 1 "EMAIL & HTML FILE MANAGEMENT - COMPLETED"
}

# 190. END
# #######################################################################################################

# #######################################################################################################
# clear down variables 	

	foreach var [info vars] {
		catch {
			if {$var != "software_version" && $var != "software_release_date"} {
				if { [catch { 
					unset $var  
				} fid] } {
					dmdebug 3 "error in unsetting variable >>$var<<"
				} 
			}
		}
	}

# #######################################################################################################
# #######################################################################################################

puts "\n------completed the jazzed dailymail program - v$software_version ($software_release_date) - thanks for your custom"
puts "------completed at [clock format [clock seconds] -format {%H:%M:%S} -gmt true] \n\n"

catch { unset software_version software_release_date }
exit
